/* IMPORT */
import { CLASS } from './constants.js';
import { memoize, toKebabCase } from './utils.js';
const Highlight = globalThis.Highlight || (Set);
const HighlightsRegistry = globalThis.CSS?.highlights || new Map();
const HighlightsSupported = !!globalThis.CSS?.highlights;
/* MAIN */
const Highlights = {
    /* API */
    getStyle: memoize(() => {
        const style = document.createElement('style');
        style.setAttribute('type', 'text/css');
        style.setAttribute(`data-${CLASS}`, '');
        document.head.append(style);
        return style;
    }),
    getHighlights: (() => {
        let highlightsCounter = 0;
        return memoize((identifier, styles) => {
            const highlight = new Highlight();
            const highlightName = `${CLASS}-${highlightsCounter++}`;
            HighlightsRegistry.set(highlightName, highlight);
            const highlightsStyle = Highlights.getStyle();
            const highlightRule = `::highlight(${highlightName}){${Object.entries(styles).filter(([key, value]) => !!value).map(([key, value]) => `${toKebabCase(key)}:${value};`).join('')}}`;
            highlightsStyle.innerText += highlightRule;
            return highlight;
        });
    })(),
    isSupported: () => {
        return HighlightsSupported;
    }
};
/* EXPORT */
export default Highlights;
